#!/bin/bash

# Extract results
(
  cd eval-sec5
  tar xf results-eval.tar.xz
)

# Setup benchmarks directory
(
  cd eval-sec5/benchmarks
  tar xf FP-25.tar.xz
  tar xf FP.tar.xz
)

# Setup solvers diretory
(
  cd solvers
  tar xf CVC4-base.tar.xz
  tar xf CVC4-ext.tar.xz
  tar xf CVC4-synth.tar.xz
  tar xf Z3.tar.xz
)

# Setup synthesis directory
(
  cd synthesis
  geninp="$(pwd)/scripts/gen-input-problems.py"
  gencand="$(pwd)/scripts/gen-input-candidates.py"

  dir=full_io_spec

  [ -e "$dir" ] && echo "directory '$dir' already exists" && exit 1

  # Extract archives
  tar xf full_io_spec_35_45_46.tar.gz

  for fmt in "3 5" "4 5" "4 6"; do
    eb=$(echo $fmt | cut -d ' ' -f 1)
    sb=$(echo $fmt | cut -d ' ' -f 2)
    basedir="$(pwd)/$dir/${eb}_${sb}"
    mkdir -p "${basedir}"

    for rm in RNE RTZ RTN RTP RNA; do
      outdir="$basedir/$rm"
      mkdir -p "$outdir"

      $geninp scripts/relations.txt scripts/operators.txt --rm $rm --eb $eb --sb $sb "$outdir"
      $geninp scripts/relations_fma.txt scripts/operators_fma.txt --rm $rm --eb $eb --sb $sb "$outdir"
    done

    mkdir -p "$basedir/simple"
    $geninp scripts/relations_simple.txt scripts/operators_simple.txt --rm $rm --eb $eb --sb $sb "$basedir/simple"

    $gencand --eb 3 --sb 5 scripts/tasks.csv "$(pwd)/$dir"
  done

  for cand in $(find "$dir" -name 'input_candidates.smt2'); do
    mv "$cand" "$(dirname "$cand")/final_solution.smt2"
  done

  # Remove redundant directories for rounding mode independent operators. Keep
  # RNE only.
  rm -r full_io_spec/{3_5,4_5,4_6}/{RNA,RTN,RTZ,RTP}/{fp.rem,fp.abs,fp.neg}
)

# Setup verification directory
(
  cd verification
  tar xf benchmarks.tar.xz
  tar xf logfiles.tar.xz
  tar xf verify-enum.tar.xz
)
