#!/usr/bin/env python

import argparse
import csv
import os
import sys


ROUNDING_MODES = ['RNE', 'RNA', 'RTP', 'RTN', 'RTZ']


def die(msg):
    print('[error] gen-input-problems.py: {}'.format(msg))
    sys.exit(1)


def main():
    ap = argparse.ArgumentParser()
    ap.add_argument('--eb', default=3, type=int)
    ap.add_argument('--sb', default=5, type=int)
    ap.add_argument('--min', default=False, action='store_true')
    ap.add_argument('csv')
    ap.add_argument('outdir')

    args = ap.parse_args()

    if not os.path.isfile(args.csv):
        die("IC candidates file '{}' does not exist".format(args.csv))
    if not os.path.exists(args.outdir):
        die("Output directory does not exist")

    fp_sort = '(_ FloatingPoint {} {})'.format(args.eb, args.sb)

    with open(args.csv) as file:
        reader = csv.reader(file, delimiter=',')

        header = next(reader, None)
        for row in reader:
            ic = row[7]
            minic = row[13]
            rm_tested = row[6]
            op = row[1]
            op_arity = int(row[3])
            rel = row[4]
            rel_arity = int(row[5])
            variant = row[2]
            if 'define-fun' not in ic:
                continue
            if 'define-fun' in minic and args.min:
                ic = minic

            if variant:
                variant = int(variant)
                assert variant > 0
                assert variant < 4
                rel = '{}{}'.format(rel, int(variant) - 1)

            ic = ic.replace('FP', fp_sort)

            if op:
                for rm in ROUNDING_MODES:
                    if rm_tested != 'All' and rm not in rm_tested:
                        continue
                    fmt = '{}_{}'.format(args.eb, args.sb)
                    path = os.path.join(args.outdir,fmt,rm,op,rel)

                    if not os.path.exists(path):
                        die('Directory {} does not exist.'.format(path))

                    out_file = os.path.join(path, 'input_candidates.smt2')
                    with open(out_file, 'w') as f:
                        f.write(ic)

            # Simple case without operator
            else:
                assert rm_tested == 'All'
                fmt = '{}_{}'.format(args.eb, args.sb)
                path = os.path.join(args.outdir,fmt,'simple',rel)

                if not os.path.exists(path):
                    die('Directory {} does not exist.'.format(path))

                out_file = os.path.join(path, 'input_candidates.smt2')
                with open(out_file, 'w') as f:
                    f.write(ic)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print()
        sys.exit('Interrupted')
