Verifying Invertibility Conditions
===================================

* `benchmarks/`: Contains all benchmarks generated for verifying the
  invertibility conditions.

* `results-{cvc4,z3}/`: Cluster log files from the verification runs with CVC4 and Z3.

* `results-enum/`: Contains the cluster log files from the
  [enumeration verification](#checking-against-full-i-o-specification) run.

* `verify-enum/`: Contains the files to rerun the enumeration verification checks.

* `run-enum-checks.sh`: Script to rerun the enumeration verification checks.

* `run-verify-checks.sh`: Script to rerun the verification runs with CVC4 and Z3.

* `show-verify-results.sh`: Script to generate the numbers in the paper from
  the cluster log files.



# Checking against full I/O specification

  For floating-point formats 3/5, 4/5 and 4/6 we checked all synthesized
  invertibility conditions against the I/O specifications, i.e., the
  invertibility condition was exhaustively evaluated on all possible input
  values. This takes about ~1.6 days of CPU time.

  To perform all checks execute:

  ```
    ./run-enum-checks.sh
  ```

  If you want perform single checks you can do the following:

  For example:
  ```
    cd verify-enum/3_5/RNE/fp.abs/fp.geq/
    run_gen_ic -test
  ```

  If testing against the I/O specification was successful the script prints:

  ```
  ***** No counterexamples for candidate!!!
  ```

  If testing fails the script prints counterexamples on which the current
  candidate does not hold.

  For example, in directory `verify-enum/3_5/RNE/fp.roundToIntegral/=/` the
  check fails due to a bug in SymFPU (as described in the paper). The
  invertibility condition for this case is correct, however, CVC4 incorrectly
  claims that there are counterexamples.

  ```
  ***** Testing candidate:
  (define-fun IC ((t (_ FloatingPoint 3 5))) Bool (= t (fp.roundToIntegral R t)))

  ***** Generated new counterexamples:
  (constraint (IC (fp #b0 #b011 #b1000) ))
  (constraint (IC (fp #b1 #b011 #b0100) ))
  (constraint (IC (fp #b1 #b011 #b1100) ))
  (constraint (IC (fp #b0 #b011 #b0100) ))
  (constraint (IC (fp #b1 #b011 #b1000) ))
  (constraint (IC (fp #b0 #b011 #b1100) ))
  ```


# Verification with CVC4 and Z3 on larger FP formats

  ## Verifying Paper Results

  The following script generates the overall number of verified invertibility
  conditions in total and per FP format from the log files of the verification
  cluster runs.

  ```
    ./show-verify-results.sh
  ```

  ## Rerun verification experiments

  Rerunning the verification experiments with CVC4 and Z3 can be done with the
  following script:

  ```
    ./run-verify-checks.sh
  ```

  **[Note]** Running all checks requires 32 days of CPU time.
  
  A smaller set of benchmarks can be rerun with:

  ```
    ./run-verify-checks.sh quick
  ```

  This runs all verification checks for `fp.abs` for FP format 3/5.
